@@ -1,13 +1,18 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this file,
+# You can obtain one at http://mozilla.org/MPL/2.0/.
+
 import os
-import time
 import tempfile
 import shutil
 import subprocess
 
-from mozprofile import FirefoxProfile, Preferences
-from mozrunner import B2GDeviceRunner
+import fxos_appgen
+import gaiatest
 import mozdevice
 import moznetwork
+import mozrunner
+from mozprofile import FirefoxProfile, Preferences
 
 from .base import get_free_port, BrowserError, Browser, ExecutorBrowser
 from ..executors.executormarionette import MarionetteTestharnessExecutor, required_files
@@ -19,7 +24,7 @@ __wptrunner__ = {"product": "b2g",
                  "browser": "B2GBrowser",
                  "executor": {"testharness": "B2GMarionetteTestharnessExecutor"},
                  "browser_kwargs": "browser_kwargs",
-                 "executor_kwargs": "get_executor_kwargs",
+                 "executor_kwargs": "executor_kwargs",
                  "env_options": "env_options"}
 
 
@@ -32,7 +37,7 @@ def browser_kwargs(**kwargs):
             "no_backup": kwargs.get("b2g_no_backup", False)}
 
 
-def get_executor_kwargs(http_server_url, **kwargs):
+def executor_kwargs(http_server_url, **kwargs):
     timeout_multiplier = kwargs["timeout_multiplier"]
     if timeout_multiplier is None:
         timeout_multiplier = 2
@@ -74,13 +79,13 @@ class B2GBrowser(Browser):
         self.logger.info("Running B2G setup")
         self.backup_path = tempfile.mkdtemp()
 
-        self.logger.debug(self.backup_path)
+        self.logger.debug("Backing up device to %s"  % (self.backup_path,))
 
         if not self.no_backup:
             self.backup_dirs = [("/data/local", os.path.join(self.backup_path, "local")),
                                 ("/data/b2g/mozilla", os.path.join(self.backup_path, "profile"))]
 
-            self.backup_paths = [("/system/etc/hosts", os.path.join(self.backup_path, "etc/hosts"))]
+            self.backup_paths = [("/system/etc/hosts", os.path.join(self.backup_path, "hosts"))]
 
             for remote, local in self.backup_dirs:
                 self.device.getDirectory(remote, local)
@@ -92,25 +97,23 @@ class B2GBrowser(Browser):
 
     def start(self):
         profile = FirefoxProfile()
+
         profile.set_preferences({"dom.disable_open_during_load": False,
-                                 "marionette.logging": True,
-                                 # "dom.mozBrowserFramesEnabled": True,
-                                 # "dom.ipc.tabs.disabled": False,
-                                 # "dom.ipc.browser_frames.oop_by_default": False,
-                                 # "marionette.force-local": True,
-                                 # "dom.testing.datastore_enabled_for_hosted_apps": True
-                                 })
+                                 "marionette.defaultPrefs.enabled": True})
 
         self.logger.debug("Creating device runner")
-        self.runner = B2GDeviceRunner(profile=profile)
+        self.runner = mozrunner.B2GDeviceRunner(profile=profile)
         self.logger.debug("Starting device runner")
         self.runner.start()
-        self.wait_for_net()
         self.logger.debug("Device runner started")
 
     def setup_hosts(self):
-        hosts = ["web-platform.test", "www.web-platform.test", "www1.web-platform.test", "www2.web-platform.test",
-                 "xn--n8j6ds53lwwkrqhv28a.web-platform.test", "xn--lve-6lad.web-platform.test"]
+        hosts = ["web-platform.test",
+                 "www.web-platform.test",
+                 "www1.web-platform.test",
+                 "www2.web-platform.test",
+                 "xn--n8j6ds53lwwkrqhv28a.web-platform.test",
+                 "xn--lve-6lad.web-platform.test"]
 
         host_ip = moznetwork.get_ip()
 
@@ -118,7 +121,7 @@ class B2GBrowser(Browser):
         hosts_path = os.path.join(temp_dir, "hosts")
         remote_path = "/system/etc/hosts"
         try:
-            self.device.getFile("/etc/hosts", hosts_path)
+            self.device.getFile("/system/etc/hosts", hosts_path)
 
             with open(hosts_path, "a+") as f:
                 hosts_present = set()
@@ -158,29 +161,12 @@ class B2GBrowser(Browser):
 
         return preferences
 
-    def wait_for_net(self):
-        # TODO: limit how long we wait before we fail
-        # consider the possibility that wlan0 is not the right interface
-
-        self.logger.info("Waiting for net connection")
-
-        def has_connection():
-            try:
-                return self.device.getIP(["wlan0"]) is not None
-            except mozdevice.DMError:
-                return False
-
-        t0 = time.time()
-        timeout = 60
-        while not has_connection():
-            if time.time() - t0 > timeout:
-                self.logger.error("Waiting for net timed out")
-                raise BrowserError("Waiting for net timed out")
-            time.sleep(1)
-
     def stop(self):
         pass
 
+    def on_output(self):
+        raise NotImplementedError
+
     def cleanup(self):
         self.logger.debug("Running browser cleanup steps")
 
@@ -208,104 +194,56 @@ class B2GBrowser(Browser):
 class B2GExecutorBrowser(ExecutorBrowser):
     # The following methods are called from a different process
     def __init__(self, *args, **kwargs):
-        print "Creating B2GExecutorBrowser"
         ExecutorBrowser.__init__(self, *args, **kwargs)
-        self.device = mozdevice.DeviceManagerADB()
+
+        import sys, subprocess
+
+        self.dm = mozdevice.DeviceManagerADB()
+        self.dm.forward("tcp:%s" % self.marionette_port,
+                        "tcp:2828")
         self.executor = None
-        print "Forwarding marionette port"
-        subprocess.check_call([self.device._adbPath,
-                               'forward',
-                               'tcp:%s' % self.marionette_port,
-                               'tcp:2828'])
+        self.marionette = None
+        self.gaia_device = None
+        self.gaia_apps = None
 
     def after_connect(self, executor):
         self.executor = executor
+        self.marionette = executor.marionette
         self.executor.logger.debug("Running browser.after_connect steps")
+
+        self.gaia_device = gaiatest.GaiaDevice(marionette=executor.marionette,
+                                               manager=self.dm)
+        self.gaia_apps = gaiatest.GaiaApps(executor.marionette)
+
+        self.executor.logger.debug("Waiting for homescreen to load")
+        self.executor.logger.debug("B2G is running: %s" % self.gaia_device.is_b2g_running)
+
+        # Moved out of gaia_test temporarily
+        self.executor.logger.info("Waiting for B2G to be ready")
+        self.wait_for_b2g_ready(timeout=60)
+
         self.install_cert_app()
         self.use_cert_app()
-        self.wait_for_net()
 
     def install_cert_app(self):
-        marionette = self.executor.marionette
-        if self.device.dirExists("/data/local/webapps/certtest-app"):
-            self.executor.logger.info("certtest_app is already installed")
+        """Install the container app used to run the tests"""
+        if fxos_appgen.is_installed("CertTest App"):
+            self.executor.logger.info("CertTest App is already installed")
             return
-        self.executor.logger.info("Installing certtest_app")
-        self.device.pushFile(os.path.join(here, "b2g_setup", "certtest_app.zip"),
-                             "/data/local/certtest_app.zip")
-
-        self.executor.logger.debug("Running install script")
-        with open(os.path.join(here, "b2g_setup", "app_install.js"), "r") as f:
-            script = f.read()
-
-        marionette.set_context("chrome")
-        marionette.set_script_timeout(5000)
-        marionette.execute_async_script(script)
-        self.executor.logger.debug("Install script complete")
+        self.executor.logger.info("Installing CertTest App")
+        app_path = os.path.join(here, "b2g_setup", "certtest_app.zip")
+        fxos_appgen.install_app("CertTest App", app_path, marionette=self.marionette)
+        self.executor.logger.debug("Install complete")
 
     def use_cert_app(self):
-        marionette = self.executor.marionette
-
-        self.wait_for_homescreen(marionette)
+        """Start the app used to run the tests"""
         self.executor.logger.info("Homescreen loaded")
+        self.gaia_apps.launch("CertTest App")
 
-        marionette.set_context("content")
-
-        # app management is done in the system app
-        marionette.switch_to_frame()
-
-        self.executor.logger.debug("Loading app_management library")
-        marionette.import_script(os.path.join(here, "b2g_setup", "app_management.js"))
-        script = "GaiaApps.launchWithName('CertTest App');"
-
-        # NOTE: if the app is already launched, this doesn't launch a new app, it will return
-        # a reference to the existing app
-        self.executor.logger.info("Launching CertTest app")
-        self.cert_test_app = marionette.execute_async_script(script, script_timeout=5000)
-        if not self.cert_test_app:
-            raise Exception("Launching CertTest App failed")
-        marionette.switch_to_frame(self.cert_test_app["frame"])
-
-    def wait_for_net(self):
-        # TODO: limit how long we wait before we fail
-        # consider the possibility that wlan0 is not the right interface
-        self.executor.logger.info("Waiting for net connection")
-        def has_connection():
-            try:
-                return self.device.getIP(["wlan0"]) is not None
-            except mozdevice.DMError:
-                return False
-
-        t0 = time.time()
-        timeout = 60
-        while not has_connection():
-            if time.time() - t0 > timeout:
-                raise BrowserError("Waiting for net timed out")
-            time.sleep(1)
-
-    def wait_for_homescreen(self, marionette):
-        self.executor.logger.info("Waiting for homescreen to load")
-        marionette.set_context(marionette.CONTEXT_CONTENT)
-        marionette.execute_async_script("""
-let manager = window.wrappedJSObject.AppWindowManager || window.wrappedJSObject.WindowManager;
-let app = null;
-if (manager) {
-  app = ('getActiveApp' in manager) ? manager.getActiveApp() : manager.getCurrentDisplayedApp();
-}
-if (app) {
-  log('Already loaded home screen');
-  marionetteScriptFinished();
-} else {
-  log('waiting for mozbrowserloadend');
-  window.addEventListener('mozbrowserloadend', function loaded(aEvent) {
-    log('received mozbrowserloadend for ' + aEvent.target.src);
-    if (aEvent.target.src.indexOf('ftu') != -1 || aEvent.target.src.indexOf('homescreen') != -1) {
-      window.removeEventListener('mozbrowserloadend', loaded);
-      marionetteScriptFinished();
-    }
-  });
-}""", script_timeout=30000)
-
+    def wait_for_b2g_ready(self, timeout):
+        # Wait for the homescreen to finish loading
+        gaiatest.Wait(self.marionette, timeout).until(gaiatest.expected.element_present(
+            gaiatest.By.CSS_SELECTOR, '#homescreen[loading-state=false]'))
 
 class B2GMarionetteTestharnessExecutor(MarionetteTestharnessExecutor):
     def after_connect(self):
